#############################################################################
##
## Copyright (C) 2015 Riverbank Computing Limited.
## Copyright (C) 2006 Thorsten Marek.
## All right reserved.
##
## This file is part of PyQt.
##
## You may use this file under the terms of the GPL v2 or the revised BSD
## license as follows:
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are
## met:
##   * Redistributions of source code must retain the above copyright
##     notice, this list of conditions and the following disclaimer.
##   * Redistributions in binary form must reproduce the above copyright
##     notice, this list of conditions and the following disclaimer in
##     the documentation and/or other materials provided with the
##     distribution.
##   * Neither the name of the Riverbank Computing Limited nor the names
##     of its contributors may be used to endorse or promote products
##     derived from this software without specific prior written
##     permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
#############################################################################


import sys
import os.path

from .exceptions import NoSuchWidgetError, WidgetPluginError

if sys.hexversion >= 0x03000000:
    from .port_v3.load_plugin import load_plugin
else:
    from .port_v2.load_plugin import load_plugin


# The list of directories that are searched for widget plugins.  This is
# exposed as part of the API.
widgetPluginPath = [os.path.join(os.path.dirname(__file__), 'widget-plugins')]


MATCH = True
NO_MATCH = False
MODULE = 0
CW_FILTER = 1


class QObjectCreator(object):    
    def __init__(self, creatorPolicy):
        self._cpolicy = creatorPolicy

        self._cwFilters = []
        self._modules = self._cpolicy.createQtGuiWidgetsWrappers()

        # Get the optional plugins.
        for plugindir in widgetPluginPath:
            try:
                plugins = os.listdir(plugindir)
            except:
                plugins = []

            for filename in plugins:
                if not filename.endswith('.py'):
                    continue

                filename = os.path.join(plugindir, filename)

                plugin_globals = {
                    "MODULE": MODULE,
                    "CW_FILTER": CW_FILTER,
                    "MATCH": MATCH,
                    "NO_MATCH": NO_MATCH}

                plugin_locals = {}

                if load_plugin(filename, plugin_globals, plugin_locals):
                    pluginType = plugin_locals["pluginType"]
                    if pluginType == MODULE:
                        modinfo = plugin_locals["moduleInformation"]()
                        self._modules.append(self._cpolicy.createModuleWrapper(*modinfo))
                    elif pluginType == CW_FILTER:
                        self._cwFilters.append(plugin_locals["getFilter"]())
                    else:
                        raise WidgetPluginError("Unknown plugin type of %s" % filename)

        self._customWidgets = self._cpolicy.createCustomWidgetLoader()
        self._modules.append(self._customWidgets)

    def createQObject(self, classname, *args, **kwargs):
        # Handle regular and custom widgets.
        factory = self.findQObjectType(classname)

        if factory is None:
            # Handle scoped names, typically static factory methods.
            parts = classname.split('.')

            if len(parts) > 1:
                factory = self.findQObjectType(parts[0])

                if factory is not None:
                    for part in parts[1:]:
                        factory = getattr(factory, part, None)
                        if factory is None:
                            break

            if factory is None:
                raise NoSuchWidgetError(classname)

        return self._cpolicy.instantiate(factory, *args, **kwargs)

    def invoke(self, rname, method, args=()):
        return self._cpolicy.invoke(rname, method, args)

    def findQObjectType(self, classname):
        for module in self._modules:
            w = module.search(classname)
            if w is not None:
                return w
        return None

    def getSlot(self, obj, slotname):
        return self._cpolicy.getSlot(obj, slotname)

    def asString(self, s):
        return self._cpolicy.asString(s)

    def addCustomWidget(self, widgetClass, baseClass, module):
        for cwFilter in self._cwFilters:
            match, result = cwFilter(widgetClass, baseClass, module)
            if match:
                widgetClass, baseClass, module = result
                break

        self._customWidgets.addCustomWidget(widgetClass, baseClass, module)



# The PEP 484 type hints stub file for the QtDesigner module.
#
# Generated by SIP 4.18.1
#
# Copyright (c) 2016 Riverbank Computing Limited <info@riverbankcomputing.com>
# 
# This file is part of PyQt5.
# 
# This file may be used under the terms of the GNU General Public License
# version 3.0 as published by the Free Software Foundation and appearing in
# the file LICENSE included in the packaging of this file.  Please review the
# following information to ensure the GNU General Public License version 3.0
# requirements will be met: http://www.gnu.org/copyleft/gpl.html.
# 
# If you do not wish to use this file under the terms of the GPL version 3.0
# then you may purchase a commercial license.  For more information contact
# info@riverbankcomputing.com.
# 
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import typing
import sip

from PyQt5 import QtWidgets
from PyQt5 import QtCore

# Support for QDate, QDateTime and QTime.
import datetime

# Convenient type aliases.
PYQT_SIGNAL = typing.Union[QtCore.pyqtSignal, QtCore.pyqtBoundSignal]
PYQT_SLOT = typing.Union[typing.Callable[..., None], QtCore.pyqtBoundSignal]

# Convenient aliases for complicated OpenGL types.
PYQT_OPENGL_ARRAY = typing.Union[typing.Sequence[int], typing.Sequence[float],
        sip.Buffer, None]
PYQT_OPENGL_BOUND_ARRAY = typing.Union[typing.Sequence[int],
        typing.Sequence[float], sip.Buffer, int, None]


class QDesignerActionEditorInterface(QtWidgets.QWidget):

    def __init__(self, parent: QtWidgets.QWidget, flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> None: ...

    def setFormWindow(self, formWindow: 'QDesignerFormWindowInterface') -> None: ...
    def unmanageAction(self, action: QtWidgets.QAction) -> None: ...
    def manageAction(self, action: QtWidgets.QAction) -> None: ...
    def core(self) -> 'QDesignerFormEditorInterface': ...


class QAbstractFormBuilder(sip.simplewrapper):

    def __init__(self) -> None: ...

    def errorString(self) -> str: ...
    def workingDirectory(self) -> QtCore.QDir: ...
    def setWorkingDirectory(self, directory: QtCore.QDir) -> None: ...
    def save(self, dev: QtCore.QIODevice, widget: QtWidgets.QWidget) -> None: ...
    def load(self, device: QtCore.QIODevice, parent: typing.Optional[QtWidgets.QWidget] = ...) -> QtWidgets.QWidget: ...


class QDesignerFormEditorInterface(QtCore.QObject):

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def setActionEditor(self, actionEditor: QDesignerActionEditorInterface) -> None: ...
    def setObjectInspector(self, objectInspector: 'QDesignerObjectInspectorInterface') -> None: ...
    def setPropertyEditor(self, propertyEditor: 'QDesignerPropertyEditorInterface') -> None: ...
    def setWidgetBox(self, widgetBox: 'QDesignerWidgetBoxInterface') -> None: ...
    def actionEditor(self) -> QDesignerActionEditorInterface: ...
    def formWindowManager(self) -> 'QDesignerFormWindowManagerInterface': ...
    def objectInspector(self) -> 'QDesignerObjectInspectorInterface': ...
    def propertyEditor(self) -> 'QDesignerPropertyEditorInterface': ...
    def widgetBox(self) -> 'QDesignerWidgetBoxInterface': ...
    def topLevel(self) -> QtWidgets.QWidget: ...
    def extensionManager(self) -> 'QExtensionManager': ...


class QDesignerFormWindowInterface(QtWidgets.QWidget):

    class FeatureFlag(int): ...
    EditFeature = ... # type: 'QDesignerFormWindowInterface.FeatureFlag'
    GridFeature = ... # type: 'QDesignerFormWindowInterface.FeatureFlag'
    TabOrderFeature = ... # type: 'QDesignerFormWindowInterface.FeatureFlag'
    DefaultFeature = ... # type: 'QDesignerFormWindowInterface.FeatureFlag'

    class Feature(sip.simplewrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QDesignerFormWindowInterface.Feature', 'QDesignerFormWindowInterface.FeatureFlag']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QDesignerFormWindowInterface.Feature') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QDesignerFormWindowInterface.Feature': ...
        def __int__(self) -> int: ...

    def __init__(self, parent: typing.Optional[QtWidgets.QWidget] = ..., flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> None: ...

    def activateResourceFilePaths(self, paths: typing.Iterable[str]) -> typing.Tuple[int, str]: ...
    def formContainer(self) -> QtWidgets.QWidget: ...
    def activeResourceFilePaths(self) -> typing.List[str]: ...
    def checkContents(self) -> typing.List[str]: ...
    def objectRemoved(self, o: QtCore.QObject) -> None: ...
    def widgetRemoved(self, w: QtWidgets.QWidget) -> None: ...
    def changed(self) -> None: ...
    def activated(self, widget: QtWidgets.QWidget) -> None: ...
    def aboutToUnmanageWidget(self, widget: QtWidgets.QWidget) -> None: ...
    def widgetUnmanaged(self, widget: QtWidgets.QWidget) -> None: ...
    def widgetManaged(self, widget: QtWidgets.QWidget) -> None: ...
    def resourceFilesChanged(self) -> None: ...
    def geometryChanged(self) -> None: ...
    def selectionChanged(self) -> None: ...
    def featureChanged(self, f: 'QDesignerFormWindowInterface.Feature') -> None: ...
    def fileNameChanged(self, fileName: str) -> None: ...
    def mainContainerChanged(self, mainContainer: QtWidgets.QWidget) -> None: ...
    def setFileName(self, fileName: str) -> None: ...
    def setGrid(self, grid: QtCore.QPoint) -> None: ...
    def selectWidget(self, widget: QtWidgets.QWidget, select: bool = ...) -> None: ...
    def clearSelection(self, update: bool = ...) -> None: ...
    def setDirty(self, dirty: bool) -> None: ...
    def setFeatures(self, f: 'QDesignerFormWindowInterface.Feature') -> None: ...
    def unmanageWidget(self, widget: QtWidgets.QWidget) -> None: ...
    def manageWidget(self, widget: QtWidgets.QWidget) -> None: ...
    def removeResourceFile(self, path: str) -> None: ...
    def addResourceFile(self, path: str) -> None: ...
    def resourceFiles(self) -> typing.List[str]: ...
    def emitSelectionChanged(self) -> None: ...
    @typing.overload
    @staticmethod
    def findFormWindow(w: QtWidgets.QWidget) -> 'QDesignerFormWindowInterface': ...
    @typing.overload
    @staticmethod
    def findFormWindow(obj: QtCore.QObject) -> 'QDesignerFormWindowInterface': ...
    def isDirty(self) -> bool: ...
    def isManaged(self, widget: QtWidgets.QWidget) -> bool: ...
    def setMainContainer(self, mainContainer: QtWidgets.QWidget) -> None: ...
    def mainContainer(self) -> QtWidgets.QWidget: ...
    def grid(self) -> QtCore.QPoint: ...
    def cursor(self) -> 'QDesignerFormWindowCursorInterface': ...
    def core(self) -> QDesignerFormEditorInterface: ...
    def setIncludeHints(self, includeHints: typing.Iterable[str]) -> None: ...
    def includeHints(self) -> typing.List[str]: ...
    def setExportMacro(self, exportMacro: str) -> None: ...
    def exportMacro(self) -> str: ...
    def setPixmapFunction(self, pixmapFunction: str) -> None: ...
    def pixmapFunction(self) -> str: ...
    def setLayoutFunction(self, margin: str, spacing: str) -> None: ...
    def layoutFunction(self) -> typing.Tuple[str, str]: ...
    def setLayoutDefault(self, margin: int, spacing: int) -> None: ...
    def layoutDefault(self) -> typing.Tuple[int, int]: ...
    def setComment(self, comment: str) -> None: ...
    def comment(self) -> str: ...
    def setAuthor(self, author: str) -> None: ...
    def author(self) -> str: ...
    def hasFeature(self, f: 'QDesignerFormWindowInterface.Feature') -> bool: ...
    def features(self) -> 'QDesignerFormWindowInterface.Feature': ...
    @typing.overload
    def setContents(self, dev: QtCore.QIODevice, errorMessage: typing.Optional[str] = ...) -> bool: ...
    @typing.overload
    def setContents(self, contents: str) -> bool: ...
    def contents(self) -> str: ...
    def absoluteDir(self) -> QtCore.QDir: ...
    def fileName(self) -> str: ...


class QDesignerFormWindowCursorInterface(sip.simplewrapper):

    class MoveMode(int): ...
    MoveAnchor = ... # type: 'QDesignerFormWindowCursorInterface.MoveMode'
    KeepAnchor = ... # type: 'QDesignerFormWindowCursorInterface.MoveMode'

    class MoveOperation(int): ...
    NoMove = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Start = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    End = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Next = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Prev = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Left = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Right = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Up = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'
    Down = ... # type: 'QDesignerFormWindowCursorInterface.MoveOperation'

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerFormWindowCursorInterface') -> None: ...

    def isWidgetSelected(self, widget: QtWidgets.QWidget) -> bool: ...
    def resetWidgetProperty(self, widget: QtWidgets.QWidget, name: str) -> None: ...
    def setWidgetProperty(self, widget: QtWidgets.QWidget, name: str, value: typing.Any) -> None: ...
    def setProperty(self, name: str, value: typing.Any) -> None: ...
    def selectedWidget(self, index: int) -> QtWidgets.QWidget: ...
    def selectedWidgetCount(self) -> int: ...
    def hasSelection(self) -> bool: ...
    def widget(self, index: int) -> QtWidgets.QWidget: ...
    def widgetCount(self) -> int: ...
    def current(self) -> QtWidgets.QWidget: ...
    def setPosition(self, pos: int, mode: 'QDesignerFormWindowCursorInterface.MoveMode' = ...) -> None: ...
    def position(self) -> int: ...
    def movePosition(self, op: 'QDesignerFormWindowCursorInterface.MoveOperation', mode: 'QDesignerFormWindowCursorInterface.MoveMode' = ...) -> bool: ...
    def formWindow(self) -> QDesignerFormWindowInterface: ...


class QDesignerFormWindowManagerInterface(QtCore.QObject):

    class ActionGroup(int): ...
    StyledPreviewActionGroup = ... # type: 'QDesignerFormWindowManagerInterface.ActionGroup'

    class Action(int): ...
    CutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    CopyAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    PasteAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    DeleteAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    SelectAllAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    LowerAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    RaiseAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    UndoAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    RedoAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    HorizontalLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    VerticalLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    SplitHorizontalAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    SplitVerticalAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    GridLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    FormLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    BreakLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    AdjustSizeAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    SimplifyLayoutAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    DefaultPreviewAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'
    FormWindowSettingsDialogAction = ... # type: 'QDesignerFormWindowManagerInterface.Action'

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def showPluginDialog(self) -> None: ...
    def closeAllPreviews(self) -> None: ...
    def showPreview(self) -> None: ...
    def actionGroup(self, actionGroup: 'QDesignerFormWindowManagerInterface.ActionGroup') -> QtWidgets.QActionGroup: ...
    def action(self, action: 'QDesignerFormWindowManagerInterface.Action') -> QtWidgets.QAction: ...
    def setActiveFormWindow(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def removeFormWindow(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def addFormWindow(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def formWindowSettingsChanged(self, fw: QDesignerFormWindowInterface) -> None: ...
    def activeFormWindowChanged(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def formWindowRemoved(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def formWindowAdded(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def core(self) -> QDesignerFormEditorInterface: ...
    def createFormWindow(self, parent: typing.Optional[QtWidgets.QWidget] = ..., flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> QDesignerFormWindowInterface: ...
    def formWindow(self, index: int) -> QDesignerFormWindowInterface: ...
    def formWindowCount(self) -> int: ...
    def activeFormWindow(self) -> QDesignerFormWindowInterface: ...
    def actionSimplifyLayout(self) -> QtWidgets.QAction: ...
    def actionFormLayout(self) -> QtWidgets.QAction: ...


class QDesignerObjectInspectorInterface(QtWidgets.QWidget):

    def __init__(self, parent: QtWidgets.QWidget, flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> None: ...

    def setFormWindow(self, formWindow: QDesignerFormWindowInterface) -> None: ...
    def core(self) -> QDesignerFormEditorInterface: ...


class QDesignerPropertyEditorInterface(QtWidgets.QWidget):

    def __init__(self, parent: QtWidgets.QWidget, flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> None: ...

    def setReadOnly(self, readOnly: bool) -> None: ...
    def setPropertyValue(self, name: str, value: typing.Any, changed: bool = ...) -> None: ...
    def setObject(self, object: QtCore.QObject) -> None: ...
    def propertyChanged(self, name: str, value: typing.Any) -> None: ...
    def currentPropertyName(self) -> str: ...
    def object(self) -> QtCore.QObject: ...
    def isReadOnly(self) -> bool: ...
    def core(self) -> QDesignerFormEditorInterface: ...


class QDesignerWidgetBoxInterface(QtWidgets.QWidget):

    def __init__(self, parent: typing.Optional[QtWidgets.QWidget] = ..., flags: typing.Union[QtCore.Qt.WindowFlags, QtCore.Qt.WindowType] = ...) -> None: ...

    def save(self) -> bool: ...
    def load(self) -> bool: ...
    def fileName(self) -> str: ...
    def setFileName(self, file_name: str) -> None: ...


class QDesignerContainerExtension(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerContainerExtension') -> None: ...

    def canRemove(self, index: int) -> bool: ...
    def canAddWidget(self) -> bool: ...
    def remove(self, index: int) -> None: ...
    def insertWidget(self, index: int, widget: QtWidgets.QWidget) -> None: ...
    def addWidget(self, widget: QtWidgets.QWidget) -> None: ...
    def setCurrentIndex(self, index: int) -> None: ...
    def currentIndex(self) -> int: ...
    def widget(self, index: int) -> QtWidgets.QWidget: ...
    def __len__(self) -> int: ...
    def count(self) -> int: ...


class QDesignerCustomWidgetInterface(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerCustomWidgetInterface') -> None: ...

    def codeTemplate(self) -> str: ...
    def domXml(self) -> str: ...
    def initialize(self, core: QDesignerFormEditorInterface) -> None: ...
    def isInitialized(self) -> bool: ...
    def createWidget(self, parent: QtWidgets.QWidget) -> QtWidgets.QWidget: ...
    def isContainer(self) -> bool: ...
    def icon(self) -> QtGui.QIcon: ...
    def includeFile(self) -> str: ...
    def whatsThis(self) -> str: ...
    def toolTip(self) -> str: ...
    def group(self) -> str: ...
    def name(self) -> str: ...


class QDesignerCustomWidgetCollectionInterface(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerCustomWidgetCollectionInterface') -> None: ...

    def customWidgets(self) -> typing.Any: ...


class QAbstractExtensionFactory(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QAbstractExtensionFactory') -> None: ...

    def extension(self, object: QtCore.QObject, iid: str) -> QtCore.QObject: ...


class QExtensionFactory(QtCore.QObject, QAbstractExtensionFactory):

    def __init__(self, parent: typing.Optional['QExtensionManager'] = ...) -> None: ...

    def createExtension(self, object: QtCore.QObject, iid: str, parent: QtCore.QObject) -> QtCore.QObject: ...
    def extensionManager(self) -> 'QExtensionManager': ...
    def extension(self, object: QtCore.QObject, iid: str) -> QtCore.QObject: ...


class QAbstractExtensionManager(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QAbstractExtensionManager') -> None: ...

    def extension(self, object: QtCore.QObject, iid: str) -> QtCore.QObject: ...
    def unregisterExtensions(self, factory: QAbstractExtensionFactory, iid: str) -> None: ...
    def registerExtensions(self, factory: QAbstractExtensionFactory, iid: str) -> None: ...


class QFormBuilder(QAbstractFormBuilder):

    def __init__(self) -> None: ...

    def customWidgets(self) -> typing.List[QDesignerCustomWidgetInterface]: ...
    def setPluginPath(self, pluginPaths: typing.Iterable[str]) -> None: ...
    def addPluginPath(self, pluginPath: str) -> None: ...
    def clearPluginPaths(self) -> None: ...
    def pluginPaths(self) -> typing.List[str]: ...


class QDesignerMemberSheetExtension(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerMemberSheetExtension') -> None: ...

    def parameterNames(self, index: int) -> typing.List[QtCore.QByteArray]: ...
    def parameterTypes(self, index: int) -> typing.List[QtCore.QByteArray]: ...
    def signature(self, index: int) -> str: ...
    def declaredInClass(self, index: int) -> str: ...
    def inheritedFromWidget(self, index: int) -> bool: ...
    def isSlot(self, index: int) -> bool: ...
    def isSignal(self, index: int) -> bool: ...
    def setVisible(self, index: int, b: bool) -> None: ...
    def isVisible(self, index: int) -> bool: ...
    def setMemberGroup(self, index: int, group: str) -> None: ...
    def memberGroup(self, index: int) -> str: ...
    def memberName(self, index: int) -> str: ...
    def indexOf(self, name: str) -> int: ...
    def __len__(self) -> int: ...
    def count(self) -> int: ...


class QDesignerPropertySheetExtension(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerPropertySheetExtension') -> None: ...

    def isEnabled(self, index: int) -> bool: ...
    def setChanged(self, index: int, changed: bool) -> None: ...
    def isChanged(self, index: int) -> bool: ...
    def setProperty(self, index: int, value: typing.Any) -> None: ...
    def property(self, index: int) -> typing.Any: ...
    def setAttribute(self, index: int, b: bool) -> None: ...
    def isAttribute(self, index: int) -> bool: ...
    def setVisible(self, index: int, b: bool) -> None: ...
    def isVisible(self, index: int) -> bool: ...
    def reset(self, index: int) -> bool: ...
    def hasReset(self, index: int) -> bool: ...
    def setPropertyGroup(self, index: int, group: str) -> None: ...
    def propertyGroup(self, index: int) -> str: ...
    def propertyName(self, index: int) -> str: ...
    def indexOf(self, name: str) -> int: ...
    def __len__(self) -> int: ...
    def count(self) -> int: ...


class QExtensionManager(QtCore.QObject, QAbstractExtensionManager):

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def extension(self, object: QtCore.QObject, iid: str) -> QtCore.QObject: ...
    def unregisterExtensions(self, factory: QAbstractExtensionFactory, iid: str = ...) -> None: ...
    def registerExtensions(self, factory: QAbstractExtensionFactory, iid: str = ...) -> None: ...


class QDesignerTaskMenuExtension(sip.simplewrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, a0: 'QDesignerTaskMenuExtension') -> None: ...

    def preferredEditAction(self) -> QtWidgets.QAction: ...
    def taskActions(self) -> typing.List[QtWidgets.QAction]: ...


class QPyDesignerContainerExtension(QtCore.QObject, QDesignerContainerExtension):

    def __init__(self, parent: QtCore.QObject) -> None: ...


class QPyDesignerCustomWidgetCollectionPlugin(QtCore.QObject, QDesignerCustomWidgetCollectionInterface):

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...


class QPyDesignerCustomWidgetPlugin(QtCore.QObject, QDesignerCustomWidgetInterface):

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...


class QPyDesignerMemberSheetExtension(QtCore.QObject, QDesignerMemberSheetExtension):

    def __init__(self, parent: QtCore.QObject) -> None: ...


class QPyDesignerPropertySheetExtension(QtCore.QObject, QDesignerPropertySheetExtension):

    def __init__(self, parent: QtCore.QObject) -> None: ...


class QPyDesignerTaskMenuExtension(QtCore.QObject, QDesignerTaskMenuExtension):

    def __init__(self, parent: QtCore.QObject) -> None: ...



# The PEP 484 type hints stub file for the QtBluetooth module.
#
# Generated by SIP 4.18.1
#
# Copyright (c) 2016 Riverbank Computing Limited <info@riverbankcomputing.com>
# 
# This file is part of PyQt5.
# 
# This file may be used under the terms of the GNU General Public License
# version 3.0 as published by the Free Software Foundation and appearing in
# the file LICENSE included in the packaging of this file.  Please review the
# following information to ensure the GNU General Public License version 3.0
# requirements will be met: http://www.gnu.org/copyleft/gpl.html.
# 
# If you do not wish to use this file under the terms of the GPL version 3.0
# then you may purchase a commercial license.  For more information contact
# info@riverbankcomputing.com.
# 
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import typing
import sip

from PyQt5 import QtCore

# Support for QDate, QDateTime and QTime.
import datetime

# Convenient type aliases.
PYQT_SIGNAL = typing.Union[QtCore.pyqtSignal, QtCore.pyqtBoundSignal]
PYQT_SLOT = typing.Union[typing.Callable[..., None], QtCore.pyqtBoundSignal]


class QBluetooth(sip.simplewrapper):

    class Security(int): ...
    NoSecurity = ... # type: 'QBluetooth.Security'
    Authorization = ... # type: 'QBluetooth.Security'
    Authentication = ... # type: 'QBluetooth.Security'
    Encryption = ... # type: 'QBluetooth.Security'
    Secure = ... # type: 'QBluetooth.Security'

    class SecurityFlags(sip.wrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QBluetooth.SecurityFlags', 'QBluetooth.Security']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QBluetooth.SecurityFlags') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QBluetooth.SecurityFlags': ...
        def __int__(self) -> int: ...


class QBluetoothAddress(sip.wrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, address: int) -> None: ...
    @typing.overload
    def __init__(self, address: str) -> None: ...
    @typing.overload
    def __init__(self, other: 'QBluetoothAddress') -> None: ...

    def toString(self) -> str: ...
    def toUInt64(self) -> int: ...
    def clear(self) -> None: ...
    def isNull(self) -> bool: ...


class QBluetoothDeviceDiscoveryAgent(QtCore.QObject):

    class InquiryType(int): ...
    GeneralUnlimitedInquiry = ... # type: 'QBluetoothDeviceDiscoveryAgent.InquiryType'
    LimitedInquiry = ... # type: 'QBluetoothDeviceDiscoveryAgent.InquiryType'

    class Error(int): ...
    NoError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'
    InputOutputError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'
    PoweredOffError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'
    InvalidBluetoothAdapterError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'
    UnsupportedPlatformError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'
    UnknownError = ... # type: 'QBluetoothDeviceDiscoveryAgent.Error'

    @typing.overload
    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, deviceAdapter: QBluetoothAddress, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def canceled(self) -> None: ...
    def finished(self) -> None: ...
    def deviceDiscovered(self, info: 'QBluetoothDeviceInfo') -> None: ...
    def stop(self) -> None: ...
    def start(self) -> None: ...
    def discoveredDevices(self) -> typing.Any: ...
    def errorString(self) -> str: ...
    @typing.overload
    def error(self) -> 'QBluetoothDeviceDiscoveryAgent.Error': ...
    @typing.overload
    def error(self, error: 'QBluetoothDeviceDiscoveryAgent.Error') -> None: ...
    def isActive(self) -> bool: ...
    def setInquiryType(self, type: 'QBluetoothDeviceDiscoveryAgent.InquiryType') -> None: ...
    def inquiryType(self) -> 'QBluetoothDeviceDiscoveryAgent.InquiryType': ...


class QBluetoothDeviceInfo(sip.wrapper):

    class CoreConfiguration(int): ...
    UnknownCoreConfiguration = ... # type: 'QBluetoothDeviceInfo.CoreConfiguration'
    LowEnergyCoreConfiguration = ... # type: 'QBluetoothDeviceInfo.CoreConfiguration'
    BaseRateCoreConfiguration = ... # type: 'QBluetoothDeviceInfo.CoreConfiguration'
    BaseRateAndLowEnergyCoreConfiguration = ... # type: 'QBluetoothDeviceInfo.CoreConfiguration'

    class DataCompleteness(int): ...
    DataComplete = ... # type: 'QBluetoothDeviceInfo.DataCompleteness'
    DataIncomplete = ... # type: 'QBluetoothDeviceInfo.DataCompleteness'
    DataUnavailable = ... # type: 'QBluetoothDeviceInfo.DataCompleteness'

    class ServiceClass(int): ...
    NoService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    PositioningService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    NetworkingService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    RenderingService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    CapturingService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    ObjectTransferService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    AudioService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    TelephonyService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    InformationService = ... # type: 'QBluetoothDeviceInfo.ServiceClass'
    AllServices = ... # type: 'QBluetoothDeviceInfo.ServiceClass'

    class MinorHealthClass(int): ...
    UncategorizedHealthDevice = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthBloodPressureMonitor = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthThermometer = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthWeightScale = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthGlucoseMeter = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthPulseOximeter = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthDataDisplay = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'
    HealthStepCounter = ... # type: 'QBluetoothDeviceInfo.MinorHealthClass'

    class MinorToyClass(int): ...
    UncategorizedToy = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'
    ToyRobot = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'
    ToyVehicle = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'
    ToyDoll = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'
    ToyController = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'
    ToyGame = ... # type: 'QBluetoothDeviceInfo.MinorToyClass'

    class MinorWearableClass(int): ...
    UncategorizedWearableDevice = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'
    WearableWristWatch = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'
    WearablePager = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'
    WearableJacket = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'
    WearableHelmet = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'
    WearableGlasses = ... # type: 'QBluetoothDeviceInfo.MinorWearableClass'

    class MinorImagingClass(int): ...
    UncategorizedImagingDevice = ... # type: 'QBluetoothDeviceInfo.MinorImagingClass'
    ImageDisplay = ... # type: 'QBluetoothDeviceInfo.MinorImagingClass'
    ImageCamera = ... # type: 'QBluetoothDeviceInfo.MinorImagingClass'
    ImageScanner = ... # type: 'QBluetoothDeviceInfo.MinorImagingClass'
    ImagePrinter = ... # type: 'QBluetoothDeviceInfo.MinorImagingClass'

    class MinorPeripheralClass(int): ...
    UncategorizedPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    KeyboardPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    PointingDevicePeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    KeyboardWithPointingDevicePeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    JoystickPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    GamepadPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    RemoteControlPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    SensingDevicePeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    DigitizerTabletPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'
    CardReaderPeripheral = ... # type: 'QBluetoothDeviceInfo.MinorPeripheralClass'

    class MinorAudioVideoClass(int): ...
    UncategorizedAudioVideoDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    WearableHeadsetDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    HandsFreeDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    Microphone = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    Loudspeaker = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    Headphones = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    PortableAudioDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    CarAudio = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    SetTopBox = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    HiFiAudioDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    Vcr = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    VideoCamera = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    Camcorder = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    VideoMonitor = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    VideoDisplayAndLoudspeaker = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    VideoConferencing = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'
    GamingDevice = ... # type: 'QBluetoothDeviceInfo.MinorAudioVideoClass'

    class MinorNetworkClass(int): ...
    NetworkFullService = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorOne = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorTwo = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorThree = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorFour = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorFive = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkLoadFactorSix = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'
    NetworkNoService = ... # type: 'QBluetoothDeviceInfo.MinorNetworkClass'

    class MinorPhoneClass(int): ...
    UncategorizedPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'
    CellularPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'
    CordlessPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'
    SmartPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'
    WiredModemOrVoiceGatewayPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'
    CommonIsdnAccessPhone = ... # type: 'QBluetoothDeviceInfo.MinorPhoneClass'

    class MinorComputerClass(int): ...
    UncategorizedComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    DesktopComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    ServerComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    LaptopComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    HandheldClamShellComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    HandheldComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'
    WearableComputer = ... # type: 'QBluetoothDeviceInfo.MinorComputerClass'

    class MinorMiscellaneousClass(int): ...
    UncategorizedMiscellaneous = ... # type: 'QBluetoothDeviceInfo.MinorMiscellaneousClass'

    class MajorDeviceClass(int): ...
    MiscellaneousDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    ComputerDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    PhoneDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    LANAccessDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    AudioVideoDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    PeripheralDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    ImagingDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    WearableDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    ToyDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    HealthDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'
    UncategorizedDevice = ... # type: 'QBluetoothDeviceInfo.MajorDeviceClass'

    class ServiceClasses(sip.wrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QBluetoothDeviceInfo.ServiceClasses', 'QBluetoothDeviceInfo.ServiceClass']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QBluetoothDeviceInfo.ServiceClasses') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QBluetoothDeviceInfo.ServiceClasses': ...
        def __int__(self) -> int: ...

    class CoreConfigurations(sip.wrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QBluetoothDeviceInfo.CoreConfigurations', 'QBluetoothDeviceInfo.CoreConfiguration']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QBluetoothDeviceInfo.CoreConfigurations') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QBluetoothDeviceInfo.CoreConfigurations': ...
        def __int__(self) -> int: ...

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, address: QBluetoothAddress, name: str, classOfDevice: int) -> None: ...
    @typing.overload
    def __init__(self, uuid: 'QBluetoothUuid', name: str, classOfDevice: int) -> None: ...
    @typing.overload
    def __init__(self, other: 'QBluetoothDeviceInfo') -> None: ...

    def deviceUuid(self) -> 'QBluetoothUuid': ...
    def setDeviceUuid(self, uuid: 'QBluetoothUuid') -> None: ...
    def coreConfigurations(self) -> 'QBluetoothDeviceInfo.CoreConfigurations': ...
    def setCoreConfigurations(self, coreConfigs: 'QBluetoothDeviceInfo.CoreConfigurations') -> None: ...
    def serviceUuidsCompleteness(self) -> 'QBluetoothDeviceInfo.DataCompleteness': ...
    def serviceUuids(self) -> typing.Tuple[typing.List['QBluetoothUuid'], 'QBluetoothDeviceInfo.DataCompleteness']: ...
    def setServiceUuids(self, uuids: typing.Any, completeness: 'QBluetoothDeviceInfo.DataCompleteness') -> None: ...
    def setRssi(self, signal: int) -> None: ...
    def rssi(self) -> int: ...
    def minorDeviceClass(self) -> int: ...
    def majorDeviceClass(self) -> 'QBluetoothDeviceInfo.MajorDeviceClass': ...
    def serviceClasses(self) -> 'QBluetoothDeviceInfo.ServiceClasses': ...
    def name(self) -> str: ...
    def address(self) -> QBluetoothAddress: ...
    def setCached(self, cached: bool) -> None: ...
    def isCached(self) -> bool: ...
    def isValid(self) -> bool: ...


class QBluetoothHostInfo(sip.wrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, other: 'QBluetoothHostInfo') -> None: ...

    def setName(self, name: str) -> None: ...
    def name(self) -> str: ...
    def setAddress(self, address: QBluetoothAddress) -> None: ...
    def address(self) -> QBluetoothAddress: ...


class QBluetoothLocalDevice(QtCore.QObject):

    class Error(int): ...
    NoError = ... # type: 'QBluetoothLocalDevice.Error'
    PairingError = ... # type: 'QBluetoothLocalDevice.Error'
    UnknownError = ... # type: 'QBluetoothLocalDevice.Error'

    class HostMode(int): ...
    HostPoweredOff = ... # type: 'QBluetoothLocalDevice.HostMode'
    HostConnectable = ... # type: 'QBluetoothLocalDevice.HostMode'
    HostDiscoverable = ... # type: 'QBluetoothLocalDevice.HostMode'
    HostDiscoverableLimitedInquiry = ... # type: 'QBluetoothLocalDevice.HostMode'

    class Pairing(int): ...
    Unpaired = ... # type: 'QBluetoothLocalDevice.Pairing'
    Paired = ... # type: 'QBluetoothLocalDevice.Pairing'
    AuthorizedPaired = ... # type: 'QBluetoothLocalDevice.Pairing'

    @typing.overload
    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, address: QBluetoothAddress, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def deviceDisconnected(self, address: QBluetoothAddress) -> None: ...
    def deviceConnected(self, address: QBluetoothAddress) -> None: ...
    def error(self, error: 'QBluetoothLocalDevice.Error') -> None: ...
    def pairingDisplayConfirmation(self, address: QBluetoothAddress, pin: str) -> None: ...
    def pairingDisplayPinCode(self, address: QBluetoothAddress, pin: str) -> None: ...
    def pairingFinished(self, address: QBluetoothAddress, pairing: 'QBluetoothLocalDevice.Pairing') -> None: ...
    def hostModeStateChanged(self, state: 'QBluetoothLocalDevice.HostMode') -> None: ...
    def pairingConfirmation(self, confirmation: bool) -> None: ...
    def connectedDevices(self) -> typing.Any: ...
    @staticmethod
    def allDevices() -> typing.Any: ...
    def address(self) -> QBluetoothAddress: ...
    def name(self) -> str: ...
    def powerOn(self) -> None: ...
    def hostMode(self) -> 'QBluetoothLocalDevice.HostMode': ...
    def setHostMode(self, mode: 'QBluetoothLocalDevice.HostMode') -> None: ...
    def pairingStatus(self, address: QBluetoothAddress) -> 'QBluetoothLocalDevice.Pairing': ...
    def requestPairing(self, address: QBluetoothAddress, pairing: 'QBluetoothLocalDevice.Pairing') -> None: ...
    def isValid(self) -> bool: ...


class QBluetoothServer(QtCore.QObject):

    class Error(int): ...
    NoError = ... # type: 'QBluetoothServer.Error'
    UnknownError = ... # type: 'QBluetoothServer.Error'
    PoweredOffError = ... # type: 'QBluetoothServer.Error'
    InputOutputError = ... # type: 'QBluetoothServer.Error'
    ServiceAlreadyRegisteredError = ... # type: 'QBluetoothServer.Error'
    UnsupportedProtocolError = ... # type: 'QBluetoothServer.Error'

    def __init__(self, serverType: 'QBluetoothServiceInfo.Protocol', parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def newConnection(self) -> None: ...
    @typing.overload
    def error(self) -> 'QBluetoothServer.Error': ...
    @typing.overload
    def error(self, a0: 'QBluetoothServer.Error') -> None: ...
    def serverType(self) -> 'QBluetoothServiceInfo.Protocol': ...
    def securityFlags(self) -> QBluetooth.SecurityFlags: ...
    def setSecurityFlags(self, security: typing.Union[QBluetooth.SecurityFlags, QBluetooth.Security]) -> None: ...
    def serverPort(self) -> int: ...
    def serverAddress(self) -> QBluetoothAddress: ...
    def nextPendingConnection(self) -> 'QBluetoothSocket': ...
    def hasPendingConnections(self) -> bool: ...
    def maxPendingConnections(self) -> int: ...
    def setMaxPendingConnections(self, numConnections: int) -> None: ...
    def isListening(self) -> bool: ...
    @typing.overload
    def listen(self, address: QBluetoothAddress = ..., port: int = ...) -> bool: ...
    @typing.overload
    def listen(self, uuid: 'QBluetoothUuid', serviceName: str = ...) -> 'QBluetoothServiceInfo': ...
    def close(self) -> None: ...


class QBluetoothServiceDiscoveryAgent(QtCore.QObject):

    class DiscoveryMode(int): ...
    MinimalDiscovery = ... # type: 'QBluetoothServiceDiscoveryAgent.DiscoveryMode'
    FullDiscovery = ... # type: 'QBluetoothServiceDiscoveryAgent.DiscoveryMode'

    class Error(int): ...
    NoError = ... # type: 'QBluetoothServiceDiscoveryAgent.Error'
    InputOutputError = ... # type: 'QBluetoothServiceDiscoveryAgent.Error'
    PoweredOffError = ... # type: 'QBluetoothServiceDiscoveryAgent.Error'
    InvalidBluetoothAdapterError = ... # type: 'QBluetoothServiceDiscoveryAgent.Error'
    UnknownError = ... # type: 'QBluetoothServiceDiscoveryAgent.Error'

    @typing.overload
    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, deviceAdapter: QBluetoothAddress, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def canceled(self) -> None: ...
    def finished(self) -> None: ...
    def serviceDiscovered(self, info: 'QBluetoothServiceInfo') -> None: ...
    def clear(self) -> None: ...
    def stop(self) -> None: ...
    def start(self, mode: 'QBluetoothServiceDiscoveryAgent.DiscoveryMode' = ...) -> None: ...
    def remoteAddress(self) -> QBluetoothAddress: ...
    def setRemoteAddress(self, address: QBluetoothAddress) -> bool: ...
    def uuidFilter(self) -> typing.List['QBluetoothUuid']: ...
    @typing.overload
    def setUuidFilter(self, uuids: typing.Iterable['QBluetoothUuid']) -> None: ...
    @typing.overload
    def setUuidFilter(self, uuid: 'QBluetoothUuid') -> None: ...
    def discoveredServices(self) -> typing.Any: ...
    def errorString(self) -> str: ...
    @typing.overload
    def error(self) -> 'QBluetoothServiceDiscoveryAgent.Error': ...
    @typing.overload
    def error(self, error: 'QBluetoothServiceDiscoveryAgent.Error') -> None: ...
    def isActive(self) -> bool: ...


class QBluetoothServiceInfo(sip.wrapper):

    class Protocol(int): ...
    UnknownProtocol = ... # type: 'QBluetoothServiceInfo.Protocol'
    L2capProtocol = ... # type: 'QBluetoothServiceInfo.Protocol'
    RfcommProtocol = ... # type: 'QBluetoothServiceInfo.Protocol'

    class AttributeId(int): ...
    ServiceRecordHandle = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceClassIds = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceRecordState = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceId = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ProtocolDescriptorList = ... # type: 'QBluetoothServiceInfo.AttributeId'
    BrowseGroupList = ... # type: 'QBluetoothServiceInfo.AttributeId'
    LanguageBaseAttributeIdList = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceInfoTimeToLive = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceAvailability = ... # type: 'QBluetoothServiceInfo.AttributeId'
    BluetoothProfileDescriptorList = ... # type: 'QBluetoothServiceInfo.AttributeId'
    DocumentationUrl = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ClientExecutableUrl = ... # type: 'QBluetoothServiceInfo.AttributeId'
    IconUrl = ... # type: 'QBluetoothServiceInfo.AttributeId'
    AdditionalProtocolDescriptorList = ... # type: 'QBluetoothServiceInfo.AttributeId'
    PrimaryLanguageBase = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceName = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceDescription = ... # type: 'QBluetoothServiceInfo.AttributeId'
    ServiceProvider = ... # type: 'QBluetoothServiceInfo.AttributeId'

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, other: 'QBluetoothServiceInfo') -> None: ...

    def serviceClassUuids(self) -> typing.List['QBluetoothUuid']: ...
    def serviceUuid(self) -> 'QBluetoothUuid': ...
    def setServiceUuid(self, uuid: 'QBluetoothUuid') -> None: ...
    def serviceAvailability(self) -> int: ...
    def setServiceAvailability(self, availability: int) -> None: ...
    def serviceProvider(self) -> str: ...
    def setServiceProvider(self, provider: str) -> None: ...
    def serviceDescription(self) -> str: ...
    def setServiceDescription(self, description: str) -> None: ...
    def serviceName(self) -> str: ...
    def setServiceName(self, name: str) -> None: ...
    @typing.overload
    def setAttribute(self, attributeId: int, value: 'QBluetoothUuid') -> None: ...
    @typing.overload
    def setAttribute(self, attributeId: int, value: typing.Iterable[typing.Any]) -> None: ...
    @typing.overload
    def setAttribute(self, attributeId: int, value: typing.Any) -> None: ...
    def unregisterService(self) -> bool: ...
    def registerService(self, localAdapter: QBluetoothAddress = ...) -> bool: ...
    def isRegistered(self) -> bool: ...
    def protocolDescriptor(self, protocol: 'QBluetoothUuid.ProtocolUuid') -> typing.List[typing.Any]: ...
    def serverChannel(self) -> int: ...
    def protocolServiceMultiplexer(self) -> int: ...
    def socketProtocol(self) -> 'QBluetoothServiceInfo.Protocol': ...
    def removeAttribute(self, attributeId: int) -> None: ...
    def contains(self, attributeId: int) -> bool: ...
    def attributes(self) -> typing.List[int]: ...
    def attribute(self, attributeId: int) -> typing.Any: ...
    def device(self) -> QBluetoothDeviceInfo: ...
    def setDevice(self, info: QBluetoothDeviceInfo) -> None: ...
    def isComplete(self) -> bool: ...
    def isValid(self) -> bool: ...


class QBluetoothSocket(QtCore.QIODevice):

    class SocketError(int): ...
    NoSocketError = ... # type: 'QBluetoothSocket.SocketError'
    UnknownSocketError = ... # type: 'QBluetoothSocket.SocketError'
    HostNotFoundError = ... # type: 'QBluetoothSocket.SocketError'
    ServiceNotFoundError = ... # type: 'QBluetoothSocket.SocketError'
    NetworkError = ... # type: 'QBluetoothSocket.SocketError'
    UnsupportedProtocolError = ... # type: 'QBluetoothSocket.SocketError'
    OperationError = ... # type: 'QBluetoothSocket.SocketError'

    class SocketState(int): ...
    UnconnectedState = ... # type: 'QBluetoothSocket.SocketState'
    ServiceLookupState = ... # type: 'QBluetoothSocket.SocketState'
    ConnectingState = ... # type: 'QBluetoothSocket.SocketState'
    ConnectedState = ... # type: 'QBluetoothSocket.SocketState'
    BoundState = ... # type: 'QBluetoothSocket.SocketState'
    ClosingState = ... # type: 'QBluetoothSocket.SocketState'
    ListeningState = ... # type: 'QBluetoothSocket.SocketState'

    @typing.overload
    def __init__(self, socketType: QBluetoothServiceInfo.Protocol, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def preferredSecurityFlags(self) -> QBluetooth.SecurityFlags: ...
    def setPreferredSecurityFlags(self, flags: typing.Union[QBluetooth.SecurityFlags, QBluetooth.Security]) -> None: ...
    def doDeviceDiscovery(self, service: QBluetoothServiceInfo, openMode: typing.Union[QtCore.QIODevice.OpenMode, QtCore.QIODevice.OpenModeFlag]) -> None: ...
    def setSocketError(self, error: 'QBluetoothSocket.SocketError') -> None: ...
    def setSocketState(self, state: 'QBluetoothSocket.SocketState') -> None: ...
    def writeData(self, data: bytes) -> int: ...
    def readData(self, maxlen: int) -> bytes: ...
    def stateChanged(self, state: 'QBluetoothSocket.SocketState') -> None: ...
    def disconnected(self) -> None: ...
    def connected(self) -> None: ...
    def errorString(self) -> str: ...
    @typing.overload
    def error(self) -> 'QBluetoothSocket.SocketError': ...
    @typing.overload
    def error(self, error: 'QBluetoothSocket.SocketError') -> None: ...
    def state(self) -> 'QBluetoothSocket.SocketState': ...
    def socketType(self) -> QBluetoothServiceInfo.Protocol: ...
    def socketDescriptor(self) -> int: ...
    def setSocketDescriptor(self, socketDescriptor: int, socketType: QBluetoothServiceInfo.Protocol, state: 'QBluetoothSocket.SocketState' = ..., mode: typing.Union[QtCore.QIODevice.OpenMode, QtCore.QIODevice.OpenModeFlag] = ...) -> bool: ...
    def peerPort(self) -> int: ...
    def peerAddress(self) -> QBluetoothAddress: ...
    def peerName(self) -> str: ...
    def localPort(self) -> int: ...
    def localAddress(self) -> QBluetoothAddress: ...
    def localName(self) -> str: ...
    def disconnectFromService(self) -> None: ...
    @typing.overload
    def connectToService(self, service: QBluetoothServiceInfo, mode: typing.Union[QtCore.QIODevice.OpenMode, QtCore.QIODevice.OpenModeFlag] = ...) -> None: ...
    @typing.overload
    def connectToService(self, address: QBluetoothAddress, uuid: 'QBluetoothUuid', mode: typing.Union[QtCore.QIODevice.OpenMode, QtCore.QIODevice.OpenModeFlag] = ...) -> None: ...
    @typing.overload
    def connectToService(self, address: QBluetoothAddress, port: int, mode: typing.Union[QtCore.QIODevice.OpenMode, QtCore.QIODevice.OpenModeFlag] = ...) -> None: ...
    def canReadLine(self) -> bool: ...
    def bytesToWrite(self) -> int: ...
    def bytesAvailable(self) -> int: ...
    def isSequential(self) -> bool: ...
    def close(self) -> None: ...
    def abort(self) -> None: ...


class QBluetoothTransferManager(QtCore.QObject):

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def finished(self, reply: 'QBluetoothTransferReply') -> None: ...
    def put(self, request: 'QBluetoothTransferRequest', data: QtCore.QIODevice) -> 'QBluetoothTransferReply': ...


class QBluetoothTransferReply(QtCore.QObject):

    class TransferError(int): ...
    NoError = ... # type: 'QBluetoothTransferReply.TransferError'
    UnknownError = ... # type: 'QBluetoothTransferReply.TransferError'
    FileNotFoundError = ... # type: 'QBluetoothTransferReply.TransferError'
    HostNotFoundError = ... # type: 'QBluetoothTransferReply.TransferError'
    UserCanceledTransferError = ... # type: 'QBluetoothTransferReply.TransferError'
    IODeviceNotReadableError = ... # type: 'QBluetoothTransferReply.TransferError'
    ResourceBusyError = ... # type: 'QBluetoothTransferReply.TransferError'
    SessionError = ... # type: 'QBluetoothTransferReply.TransferError'

    def __init__(self, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def setRequest(self, request: 'QBluetoothTransferRequest') -> None: ...
    def setManager(self, manager: QBluetoothTransferManager) -> None: ...
    def transferProgress(self, bytesTransferred: int, bytesTotal: int) -> None: ...
    def finished(self, a0: 'QBluetoothTransferReply') -> None: ...
    def abort(self) -> None: ...
    def request(self) -> 'QBluetoothTransferRequest': ...
    def errorString(self) -> str: ...
    @typing.overload
    def error(self) -> 'QBluetoothTransferReply.TransferError': ...
    @typing.overload
    def error(self, lastError: 'QBluetoothTransferReply.TransferError') -> None: ...
    def manager(self) -> QBluetoothTransferManager: ...
    def isRunning(self) -> bool: ...
    def isFinished(self) -> bool: ...


class QBluetoothTransferRequest(sip.wrapper):

    class Attribute(int): ...
    DescriptionAttribute = ... # type: 'QBluetoothTransferRequest.Attribute'
    TimeAttribute = ... # type: 'QBluetoothTransferRequest.Attribute'
    TypeAttribute = ... # type: 'QBluetoothTransferRequest.Attribute'
    LengthAttribute = ... # type: 'QBluetoothTransferRequest.Attribute'
    NameAttribute = ... # type: 'QBluetoothTransferRequest.Attribute'

    @typing.overload
    def __init__(self, address: QBluetoothAddress = ...) -> None: ...
    @typing.overload
    def __init__(self, other: 'QBluetoothTransferRequest') -> None: ...

    def address(self) -> QBluetoothAddress: ...
    def setAttribute(self, code: 'QBluetoothTransferRequest.Attribute', value: typing.Any) -> None: ...
    def attribute(self, code: 'QBluetoothTransferRequest.Attribute', defaultValue: typing.Any = ...) -> typing.Any: ...


class QBluetoothUuid(QtCore.QUuid):

    class DescriptorType(int): ...
    UnknownDescriptorType = ... # type: 'QBluetoothUuid.DescriptorType'
    CharacteristicExtendedProperties = ... # type: 'QBluetoothUuid.DescriptorType'
    CharacteristicUserDescription = ... # type: 'QBluetoothUuid.DescriptorType'
    ClientCharacteristicConfiguration = ... # type: 'QBluetoothUuid.DescriptorType'
    ServerCharacteristicConfiguration = ... # type: 'QBluetoothUuid.DescriptorType'
    CharacteristicPresentationFormat = ... # type: 'QBluetoothUuid.DescriptorType'
    CharacteristicAggregateFormat = ... # type: 'QBluetoothUuid.DescriptorType'
    ValidRange = ... # type: 'QBluetoothUuid.DescriptorType'
    ExternalReportReference = ... # type: 'QBluetoothUuid.DescriptorType'
    ReportReference = ... # type: 'QBluetoothUuid.DescriptorType'
    EnvironmentalSensingConfiguration = ... # type: 'QBluetoothUuid.DescriptorType'
    EnvironmentalSensingMeasurement = ... # type: 'QBluetoothUuid.DescriptorType'
    EnvironmentalSensingTriggerSetting = ... # type: 'QBluetoothUuid.DescriptorType'

    class CharacteristicType(int): ...
    DeviceName = ... # type: 'QBluetoothUuid.CharacteristicType'
    Appearance = ... # type: 'QBluetoothUuid.CharacteristicType'
    PeripheralPrivacyFlag = ... # type: 'QBluetoothUuid.CharacteristicType'
    ReconnectionAddress = ... # type: 'QBluetoothUuid.CharacteristicType'
    PeripheralPreferredConnectionParameters = ... # type: 'QBluetoothUuid.CharacteristicType'
    ServiceChanged = ... # type: 'QBluetoothUuid.CharacteristicType'
    AlertLevel = ... # type: 'QBluetoothUuid.CharacteristicType'
    TxPowerLevel = ... # type: 'QBluetoothUuid.CharacteristicType'
    DateTime = ... # type: 'QBluetoothUuid.CharacteristicType'
    DayOfWeek = ... # type: 'QBluetoothUuid.CharacteristicType'
    DayDateTime = ... # type: 'QBluetoothUuid.CharacteristicType'
    ExactTime256 = ... # type: 'QBluetoothUuid.CharacteristicType'
    DSTOffset = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeZone = ... # type: 'QBluetoothUuid.CharacteristicType'
    LocalTimeInformation = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeWithDST = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeAccuracy = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeSource = ... # type: 'QBluetoothUuid.CharacteristicType'
    ReferenceTimeInformation = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeUpdateControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    TimeUpdateState = ... # type: 'QBluetoothUuid.CharacteristicType'
    GlucoseMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    BatteryLevel = ... # type: 'QBluetoothUuid.CharacteristicType'
    TemperatureMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    TemperatureType = ... # type: 'QBluetoothUuid.CharacteristicType'
    IntermediateTemperature = ... # type: 'QBluetoothUuid.CharacteristicType'
    MeasurementInterval = ... # type: 'QBluetoothUuid.CharacteristicType'
    BootKeyboardInputReport = ... # type: 'QBluetoothUuid.CharacteristicType'
    SystemID = ... # type: 'QBluetoothUuid.CharacteristicType'
    ModelNumberString = ... # type: 'QBluetoothUuid.CharacteristicType'
    SerialNumberString = ... # type: 'QBluetoothUuid.CharacteristicType'
    FirmwareRevisionString = ... # type: 'QBluetoothUuid.CharacteristicType'
    HardwareRevisionString = ... # type: 'QBluetoothUuid.CharacteristicType'
    SoftwareRevisionString = ... # type: 'QBluetoothUuid.CharacteristicType'
    ManufacturerNameString = ... # type: 'QBluetoothUuid.CharacteristicType'
    IEEE1107320601RegulatoryCertificationDataList = ... # type: 'QBluetoothUuid.CharacteristicType'
    CurrentTime = ... # type: 'QBluetoothUuid.CharacteristicType'
    ScanRefresh = ... # type: 'QBluetoothUuid.CharacteristicType'
    BootKeyboardOutputReport = ... # type: 'QBluetoothUuid.CharacteristicType'
    BootMouseInputReport = ... # type: 'QBluetoothUuid.CharacteristicType'
    GlucoseMeasurementContext = ... # type: 'QBluetoothUuid.CharacteristicType'
    BloodPressureMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    IntermediateCuffPressure = ... # type: 'QBluetoothUuid.CharacteristicType'
    HeartRateMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    BodySensorLocation = ... # type: 'QBluetoothUuid.CharacteristicType'
    HeartRateControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    AlertStatus = ... # type: 'QBluetoothUuid.CharacteristicType'
    RingerControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    RingerSetting = ... # type: 'QBluetoothUuid.CharacteristicType'
    AlertCategoryIDBitMask = ... # type: 'QBluetoothUuid.CharacteristicType'
    AlertCategoryID = ... # type: 'QBluetoothUuid.CharacteristicType'
    AlertNotificationControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    UnreadAlertStatus = ... # type: 'QBluetoothUuid.CharacteristicType'
    NewAlert = ... # type: 'QBluetoothUuid.CharacteristicType'
    SupportedNewAlertCategory = ... # type: 'QBluetoothUuid.CharacteristicType'
    SupportedUnreadAlertCategory = ... # type: 'QBluetoothUuid.CharacteristicType'
    BloodPressureFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    HIDInformation = ... # type: 'QBluetoothUuid.CharacteristicType'
    ReportMap = ... # type: 'QBluetoothUuid.CharacteristicType'
    HIDControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    Report = ... # type: 'QBluetoothUuid.CharacteristicType'
    ProtocolMode = ... # type: 'QBluetoothUuid.CharacteristicType'
    ScanIntervalWindow = ... # type: 'QBluetoothUuid.CharacteristicType'
    PnPID = ... # type: 'QBluetoothUuid.CharacteristicType'
    GlucoseFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    RecordAccessControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    RSCMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    RSCFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    SCControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    CSCMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    CSCFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    SensorLocation = ... # type: 'QBluetoothUuid.CharacteristicType'
    CyclingPowerMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    CyclingPowerVector = ... # type: 'QBluetoothUuid.CharacteristicType'
    CyclingPowerFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    CyclingPowerControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    LocationAndSpeed = ... # type: 'QBluetoothUuid.CharacteristicType'
    Navigation = ... # type: 'QBluetoothUuid.CharacteristicType'
    PositionQuality = ... # type: 'QBluetoothUuid.CharacteristicType'
    LNFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    LNControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    MagneticDeclination = ... # type: 'QBluetoothUuid.CharacteristicType'
    Elevation = ... # type: 'QBluetoothUuid.CharacteristicType'
    Pressure = ... # type: 'QBluetoothUuid.CharacteristicType'
    Temperature = ... # type: 'QBluetoothUuid.CharacteristicType'
    Humidity = ... # type: 'QBluetoothUuid.CharacteristicType'
    TrueWindSpeed = ... # type: 'QBluetoothUuid.CharacteristicType'
    TrueWindDirection = ... # type: 'QBluetoothUuid.CharacteristicType'
    ApparentWindSpeed = ... # type: 'QBluetoothUuid.CharacteristicType'
    ApparentWindDirection = ... # type: 'QBluetoothUuid.CharacteristicType'
    GustFactor = ... # type: 'QBluetoothUuid.CharacteristicType'
    PollenConcentration = ... # type: 'QBluetoothUuid.CharacteristicType'
    UVIndex = ... # type: 'QBluetoothUuid.CharacteristicType'
    Irradiance = ... # type: 'QBluetoothUuid.CharacteristicType'
    Rainfall = ... # type: 'QBluetoothUuid.CharacteristicType'
    WindChill = ... # type: 'QBluetoothUuid.CharacteristicType'
    HeatIndex = ... # type: 'QBluetoothUuid.CharacteristicType'
    DewPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    DescriptorValueChanged = ... # type: 'QBluetoothUuid.CharacteristicType'
    AerobicHeartRateLowerLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    AerobicThreshold = ... # type: 'QBluetoothUuid.CharacteristicType'
    Age = ... # type: 'QBluetoothUuid.CharacteristicType'
    AnaerobicHeartRateLowerLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    AnaerobicHeartRateUpperLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    AnaerobicThreshold = ... # type: 'QBluetoothUuid.CharacteristicType'
    AerobicHeartRateUpperLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    DateOfBirth = ... # type: 'QBluetoothUuid.CharacteristicType'
    DateOfThresholdAssessment = ... # type: 'QBluetoothUuid.CharacteristicType'
    EmailAddress = ... # type: 'QBluetoothUuid.CharacteristicType'
    FatBurnHeartRateLowerLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    FatBurnHeartRateUpperLimit = ... # type: 'QBluetoothUuid.CharacteristicType'
    FirstName = ... # type: 'QBluetoothUuid.CharacteristicType'
    FiveZoneHeartRateLimits = ... # type: 'QBluetoothUuid.CharacteristicType'
    Gender = ... # type: 'QBluetoothUuid.CharacteristicType'
    HeartRateMax = ... # type: 'QBluetoothUuid.CharacteristicType'
    Height = ... # type: 'QBluetoothUuid.CharacteristicType'
    HipCircumference = ... # type: 'QBluetoothUuid.CharacteristicType'
    LastName = ... # type: 'QBluetoothUuid.CharacteristicType'
    MaximumRecommendedHeartRate = ... # type: 'QBluetoothUuid.CharacteristicType'
    RestingHeartRate = ... # type: 'QBluetoothUuid.CharacteristicType'
    SportTypeForAerobicAnaerobicThresholds = ... # type: 'QBluetoothUuid.CharacteristicType'
    ThreeZoneHeartRateLimits = ... # type: 'QBluetoothUuid.CharacteristicType'
    TwoZoneHeartRateLimits = ... # type: 'QBluetoothUuid.CharacteristicType'
    VO2Max = ... # type: 'QBluetoothUuid.CharacteristicType'
    WaistCircumference = ... # type: 'QBluetoothUuid.CharacteristicType'
    Weight = ... # type: 'QBluetoothUuid.CharacteristicType'
    DatabaseChangeIncrement = ... # type: 'QBluetoothUuid.CharacteristicType'
    UserIndex = ... # type: 'QBluetoothUuid.CharacteristicType'
    BodyCompositionFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    BodyCompositionMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    WeightMeasurement = ... # type: 'QBluetoothUuid.CharacteristicType'
    WeightScaleFeature = ... # type: 'QBluetoothUuid.CharacteristicType'
    UserControlPoint = ... # type: 'QBluetoothUuid.CharacteristicType'
    MagneticFluxDensity2D = ... # type: 'QBluetoothUuid.CharacteristicType'
    MagneticFluxDensity3D = ... # type: 'QBluetoothUuid.CharacteristicType'
    Language = ... # type: 'QBluetoothUuid.CharacteristicType'
    BarometricPressureTrend = ... # type: 'QBluetoothUuid.CharacteristicType'

    class ServiceClassUuid(int): ...
    ServiceDiscoveryServer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BrowseGroupDescriptor = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PublicBrowseGroup = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    SerialPort = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    LANAccessUsingPPP = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    DialupNetworking = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    IrMCSync = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ObexObjectPush = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    OBEXFileTransfer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    IrMCSyncCommand = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Headset = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AudioSource = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AudioSink = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AV_RemoteControlTarget = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AdvancedAudioDistribution = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AV_RemoteControl = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AV_RemoteControlController = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HeadsetAG = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PANU = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    NAP = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GN = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    DirectPrinting = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ReferencePrinting = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ImagingResponder = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ImagingAutomaticArchive = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ImagingReferenceObjects = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Handsfree = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HandsfreeAudioGateway = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    DirectPrintingReferenceObjectsService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ReflectedUI = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BasicPrinting = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PrintingStatus = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HumanInterfaceDeviceService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HardcopyCableReplacement = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HCRPrint = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HCRScan = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    SIMAccess = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PhonebookAccessPCE = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PhonebookAccessPSE = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PhonebookAccess = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HeadsetHS = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    MessageAccessServer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    MessageNotificationServer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    MessageAccessProfile = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PnPInformation = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericNetworking = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericFileTransfer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericAudio = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericTelephony = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    VideoSource = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    VideoSink = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    VideoDistribution = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HDP = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HDPSource = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HDPSink = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BasicImage = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GNSS = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GNSSServer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Display3D = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Glasses3D = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Synchronization3D = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    MPSProfile = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    MPSService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericAccess = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    GenericAttribute = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ImmediateAlert = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    LinkLoss = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    TxPower = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    CurrentTimeService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ReferenceTimeUpdateService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    NextDSTChangeService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    Glucose = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HealthThermometer = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    DeviceInformation = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HeartRate = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    PhoneAlertStatusService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BatteryService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BloodPressure = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    AlertNotificationService = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    HumanInterfaceDevice = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ScanParameters = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    RunningSpeedAndCadence = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    CyclingSpeedAndCadence = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    CyclingPower = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    LocationAndNavigation = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    EnvironmentalSensing = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BodyComposition = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    UserData = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    WeightScale = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    BondManagement = ... # type: 'QBluetoothUuid.ServiceClassUuid'
    ContinuousGlucoseMonitoring = ... # type: 'QBluetoothUuid.ServiceClassUuid'

    class ProtocolUuid(int): ...
    Sdp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Udp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Rfcomm = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Tcp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    TcsBin = ... # type: 'QBluetoothUuid.ProtocolUuid'
    TcsAt = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Att = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Obex = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Ip = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Ftp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Http = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Wsp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Bnep = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Upnp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Hidp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    HardcopyControlChannel = ... # type: 'QBluetoothUuid.ProtocolUuid'
    HardcopyDataChannel = ... # type: 'QBluetoothUuid.ProtocolUuid'
    HardcopyNotification = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Avctp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Avdtp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    Cmtp = ... # type: 'QBluetoothUuid.ProtocolUuid'
    UdiCPlain = ... # type: 'QBluetoothUuid.ProtocolUuid'
    McapControlChannel = ... # type: 'QBluetoothUuid.ProtocolUuid'
    McapDataChannel = ... # type: 'QBluetoothUuid.ProtocolUuid'
    L2cap = ... # type: 'QBluetoothUuid.ProtocolUuid'

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, uuid: int) -> None: ...
    @typing.overload
    def __init__(self, uuid: Tuple[int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int) -> None: ...
    @typing.overload
    def __init__(self, uuid: str) -> None: ...
    @typing.overload
    def __init__(self, uuid: 'QBluetoothUuid') -> None: ...
    @typing.overload
    def __init__(self, uuid: QtCore.QUuid) -> None: ...

    @staticmethod
    def descriptorToString(uuid: 'QBluetoothUuid.DescriptorType') -> str: ...
    @staticmethod
    def characteristicToString(uuid: 'QBluetoothUuid.CharacteristicType') -> str: ...
    @staticmethod
    def protocolToString(uuid: 'QBluetoothUuid.ProtocolUuid') -> str: ...
    @staticmethod
    def serviceClassToString(uuid: 'QBluetoothUuid.ServiceClassUuid') -> str: ...
    def toUInt128(self) -> Tuple[int, int, int, int, int, int, int, int, int, int, int, int, int, int, int, int: ...
    def toUInt32(self) -> typing.Tuple[int, bool]: ...
    def toUInt16(self) -> typing.Tuple[int, bool]: ...
    def minimumSize(self) -> int: ...


class QLowEnergyCharacteristic(sip.wrapper):

    class PropertyType(int): ...
    Unknown = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    Broadcasting = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    Read = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    WriteNoResponse = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    Write = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    Notify = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    Indicate = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    WriteSigned = ... # type: 'QLowEnergyCharacteristic.PropertyType'
    ExtendedProperty = ... # type: 'QLowEnergyCharacteristic.PropertyType'

    class PropertyTypes(sip.wrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QLowEnergyCharacteristic.PropertyTypes', 'QLowEnergyCharacteristic.PropertyType']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QLowEnergyCharacteristic.PropertyTypes') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QLowEnergyCharacteristic.PropertyTypes': ...
        def __int__(self) -> int: ...

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, other: 'QLowEnergyCharacteristic') -> None: ...

    def isValid(self) -> bool: ...
    def descriptors(self) -> typing.Any: ...
    def descriptor(self, uuid: QBluetoothUuid) -> 'QLowEnergyDescriptor': ...
    def handle(self) -> int: ...
    def properties(self) -> 'QLowEnergyCharacteristic.PropertyTypes': ...
    def value(self) -> QtCore.QByteArray: ...
    def uuid(self) -> QBluetoothUuid: ...
    def name(self) -> str: ...


class QLowEnergyController(QtCore.QObject):

    class RemoteAddressType(int): ...
    PublicAddress = ... # type: 'QLowEnergyController.RemoteAddressType'
    RandomAddress = ... # type: 'QLowEnergyController.RemoteAddressType'

    class ControllerState(int): ...
    UnconnectedState = ... # type: 'QLowEnergyController.ControllerState'
    ConnectingState = ... # type: 'QLowEnergyController.ControllerState'
    ConnectedState = ... # type: 'QLowEnergyController.ControllerState'
    DiscoveringState = ... # type: 'QLowEnergyController.ControllerState'
    DiscoveredState = ... # type: 'QLowEnergyController.ControllerState'
    ClosingState = ... # type: 'QLowEnergyController.ControllerState'

    class Error(int): ...
    NoError = ... # type: 'QLowEnergyController.Error'
    UnknownError = ... # type: 'QLowEnergyController.Error'
    UnknownRemoteDeviceError = ... # type: 'QLowEnergyController.Error'
    NetworkError = ... # type: 'QLowEnergyController.Error'
    InvalidBluetoothAdapterError = ... # type: 'QLowEnergyController.Error'
    ConnectionError = ... # type: 'QLowEnergyController.Error'

    @typing.overload
    def __init__(self, remoteDevice: QBluetoothDeviceInfo, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, remoteDevice: QBluetoothAddress, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...
    @typing.overload
    def __init__(self, remoteDevice: QBluetoothAddress, localDevice: QBluetoothAddress, parent: typing.Optional[QtCore.QObject] = ...) -> None: ...

    def discoveryFinished(self) -> None: ...
    def serviceDiscovered(self, newService: QBluetoothUuid) -> None: ...
    def stateChanged(self, state: 'QLowEnergyController.ControllerState') -> None: ...
    def disconnected(self) -> None: ...
    def connected(self) -> None: ...
    def remoteName(self) -> str: ...
    def errorString(self) -> str: ...
    @typing.overload
    def error(self) -> 'QLowEnergyController.Error': ...
    @typing.overload
    def error(self, newError: 'QLowEnergyController.Error') -> None: ...
    def createServiceObject(self, service: QBluetoothUuid, parent: typing.Optional[QtCore.QObject] = ...) -> 'QLowEnergyService': ...
    def services(self) -> typing.List[QBluetoothUuid]: ...
    def discoverServices(self) -> None: ...
    def disconnectFromDevice(self) -> None: ...
    def connectToDevice(self) -> None: ...
    def setRemoteAddressType(self, type: 'QLowEnergyController.RemoteAddressType') -> None: ...
    def remoteAddressType(self) -> 'QLowEnergyController.RemoteAddressType': ...
    def state(self) -> 'QLowEnergyController.ControllerState': ...
    def remoteAddress(self) -> QBluetoothAddress: ...
    def localAddress(self) -> QBluetoothAddress: ...


class QLowEnergyDescriptor(sip.wrapper):

    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, other: 'QLowEnergyDescriptor') -> None: ...

    def type(self) -> QBluetoothUuid.DescriptorType: ...
    def name(self) -> str: ...
    def handle(self) -> int: ...
    def uuid(self) -> QBluetoothUuid: ...
    def value(self) -> QtCore.QByteArray: ...
    def isValid(self) -> bool: ...


class QLowEnergyService(QtCore.QObject):

    class WriteMode(int): ...
    WriteWithResponse = ... # type: 'QLowEnergyService.WriteMode'
    WriteWithoutResponse = ... # type: 'QLowEnergyService.WriteMode'

    class ServiceState(int): ...
    InvalidService = ... # type: 'QLowEnergyService.ServiceState'
    DiscoveryRequired = ... # type: 'QLowEnergyService.ServiceState'
    DiscoveringServices = ... # type: 'QLowEnergyService.ServiceState'
    ServiceDiscovered = ... # type: 'QLowEnergyService.ServiceState'

    class ServiceError(int): ...
    NoError = ... # type: 'QLowEnergyService.ServiceError'
    OperationError = ... # type: 'QLowEnergyService.ServiceError'
    CharacteristicWriteError = ... # type: 'QLowEnergyService.ServiceError'
    DescriptorWriteError = ... # type: 'QLowEnergyService.ServiceError'
    CharacteristicReadError = ... # type: 'QLowEnergyService.ServiceError'
    DescriptorReadError = ... # type: 'QLowEnergyService.ServiceError'
    UnknownError = ... # type: 'QLowEnergyService.ServiceError'

    class ServiceType(int): ...
    PrimaryService = ... # type: 'QLowEnergyService.ServiceType'
    IncludedService = ... # type: 'QLowEnergyService.ServiceType'

    class ServiceTypes(sip.wrapper):

        @typing.overload
        def __init__(self) -> None: ...
        @typing.overload
        def __init__(self, f: typing.Union['QLowEnergyService.ServiceTypes', 'QLowEnergyService.ServiceType']) -> None: ...
        @typing.overload
        def __init__(self, a0: 'QLowEnergyService.ServiceTypes') -> None: ...

        def __bool__(self) -> int: ...
        def __invert__(self) -> 'QLowEnergyService.ServiceTypes': ...
        def __int__(self) -> int: ...

    def descriptorRead(self, info: QLowEnergyDescriptor, value: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def characteristicRead(self, info: QLowEnergyCharacteristic, value: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def readDescriptor(self, descriptor: QLowEnergyDescriptor) -> None: ...
    def readCharacteristic(self, characteristic: QLowEnergyCharacteristic) -> None: ...
    def descriptorWritten(self, info: QLowEnergyDescriptor, value: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def characteristicWritten(self, info: QLowEnergyCharacteristic, value: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def characteristicChanged(self, info: QLowEnergyCharacteristic, value: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def stateChanged(self, newState: 'QLowEnergyService.ServiceState') -> None: ...
    def writeDescriptor(self, descriptor: QLowEnergyDescriptor, newValue: typing.Union[QtCore.QByteArray, bytes, bytearray]) -> None: ...
    def writeCharacteristic(self, characteristic: QLowEnergyCharacteristic, newValue: typing.Union[QtCore.QByteArray, bytes, bytearray], mode: 'QLowEnergyService.WriteMode' = ...) -> None: ...
    @typing.overload
    def contains(self, characteristic: QLowEnergyCharacteristic) -> bool: ...
    @typing.overload
    def contains(self, descriptor: QLowEnergyDescriptor) -> bool: ...
    @typing.overload
    def error(self) -> 'QLowEnergyService.ServiceError': ...
    @typing.overload
    def error(self, error: 'QLowEnergyService.ServiceError') -> None: ...
    def discoverDetails(self) -> None: ...
    def serviceName(self) -> str: ...
    def serviceUuid(self) -> QBluetoothUuid: ...
    def characteristics(self) -> typing.Any: ...
    def characteristic(self, uuid: QBluetoothUuid) -> QLowEnergyCharacteristic: ...
    def state(self) -> 'QLowEnergyService.ServiceState': ...
    def type(self) -> 'QLowEnergyService.ServiceTypes': ...
    def includedServices(self) -> typing.List[QBluetoothUuid]: ...
